@@ -2,11 +2,9 @@
 For detailed API information, see:
 https://developer.quickbase.com/
 """
-
 import traceback
 from typing import Any, Dict, List
 import xml.etree.ElementTree as ET
-import pprint
 try:
     import requests
 except ImportError:
@@ -16,8 +14,6 @@ from requests.exceptions import ConnectionError
 
 import time
 
-from collections import OrderedDict, namedtuple
-
 from requests.packages import urllib3 as urllib3
 
 urllib3.disable_warnings()
@@ -51,7 +47,7 @@ class QuickBaseError(QBError):
 class MissingValueException(Exception):
     """Exception raised when a value is missing from an API call. Define custome message if needed."""
 
-    def __init__(self, message="Missing value") -> None:
+    def __init__(self, message="Missing value"):
         self.message = message
         super().__init__(self.message)
 
@@ -62,8 +58,8 @@ class MissingValueException(Exception):
 class Client(object):
     """Client to the QuickBase API."""
 
-    def __init__(self, username=None, password=None, base_url='https://www.quickbase.com',
-                 timeout=90, database=None, realmhost=None, ticket=None, user_token=None):
+    def __init__(self, username=None, base_url='https://www.quickbase.com',
+                 timeout=90, database=None, realmhost=None, user_token=None):
         """Initialize a Client with user_token rather than username and password.
         """
         self.username = username
@@ -82,53 +78,16 @@ class Client(object):
         if user_token is not None:
             self.headers.update(
                 {'Authorization': f'QB-USER-TOKEN {user_token}'})
-
-        elif username and password:
-            self.username = username
-            self.password = password
-
-            try:
-                error = self.authenticate(username, password)
-
-                if error:
-                    raise Exception(error)
-
-            except Exception as e:
-                print("You must provide some credentials, preferably a user token.")
-                quit()
-
-    def authenticate(self, password, hours):
-        url = f'https://{self.realm}.quickbase.com/db/main?a=API_Authenticate&username={self.username}&password={password}&hours={hours}'
-        r = requests.post(url=url)
-
-        pprint.pp(r.text)
-
-        tree = ET.fromstring(r.content)
-
-        xml_dict = {}
-        for child in tree:
-            xml_dict.update({child.tag: child.text})
-
-        error_code = int(xml_dict.get('errcode'))
-
-        if error_code == 0:
-            token = xml_dict.get('ticket')
-            self.token = token
-            self.authenticated = True
-            return error_code
         else:
-            return int(xml_dict.get("errcode")), xml_dict.get("error")
+            print("In order to use this SDK, you must provide a Quickbase user token.")
 
-    def request(self, action, database, data, required=None):
+    def request(self, action, database, data):
         """Do a QuickBase request and return the parsed JSON response. Raises appropriate
         Error subclass on HTTP, response or QuickBase error. If fields list given,
         return dict with all fields in list (raises ResponseError if any not present),
         otherwise return parsed JSON Element.
         """
 
-        # self.headers.update({'encoding': 'UTF-8'})
-        # self.headers.update({'msInUTC': '1'})
-
         if action == "query":
             url = "https://api.quickbase.com/v1/records/query"
 
@@ -155,14 +114,6 @@ class Client(object):
 
                 pass
 
-        # response = request.content
-
-        # encoding = chardet.detect(response)['encoding']
-        # # encoding = "ascii"
-
-        # if encoding != 'utf-8':
-        #     response = response.decode(encoding, 'replace').encode('utf-8')
-
         return response
 
     def denest(self, data):
@@ -249,22 +200,26 @@ class Client(object):
                     record[field] = {'value': int(value['value'])}
         return data
 
-    def do_query(self, query=None, qid=None, qname=None, columns=None, sort=None,
-                 structured=True, num=None, skip=None, ascending=True,
+    def get_sort_list(self, fids, ascending):
+        if ascending:
+            order = 'ASC'
+        else:
+            order = 'DESC'
+
+        if fids:
+            return [{'fieldId': fid, 'order': order} for fid in fids]
+        else:
+            return []
+
+    def do_query(self, query=None, qid=None, columns=None, sort=[3],
+                 num=None, skip=None, ascending=True,
                  database=None, round_ints=False):
         """Perform query and return results (list of dicts)."""
 
-        data = OrderedDict()
         if len([q for q in (query, qid) if q]) != 1:
             raise TypeError('must specify one of query, qid')
-
-        if sort and columns:
-            for fid in sort:
-                if fid not in columns:
-                    columns.append(fid)
-
+        # regular query
         if query:
-            url = "https://api.quickbase.com/v1/records/query"
             try:
                 body = {
                     "from": database,
@@ -272,18 +227,13 @@ class Client(object):
                         column for column in (columns or [])
                     ],
                     "where": query,
-
+                    "sortBy": self.get_sort_list(sort, ascending)
                 }
-                response = self.perform_query(body=body)
 
-                res_json = response.json()
 
-                res_json['data'] = self.fix_nums(res_json['data'])
-                if sort:
-                    res_json = self.sort_response_by_fid(
-                        res_json, sort, not ascending)
+                response =  self.perform_query(body=body)
 
-                data = res_json.get("data")
+                data = self.fix_nums(response.get('data'))
 
                 if round_ints:
                     data = self.round_ints(data)
@@ -291,6 +241,8 @@ class Client(object):
                 return self.denest(data)
             except Exception as e:
                 print(f"Error during query: {traceback.format_exc()}")
+
+        # run a report, keep report columns
         if qid and columns is None:
             try:
                 url = f"https://api.quickbase.com/v1/reports/{qid}/run"
@@ -298,15 +250,7 @@ class Client(object):
                 response = self.run_report(
                     url, database=database, skip=skip, max=num)
 
-                res_json = response.json()
-
-                res_json['data'] = self.fix_nums(res_json['data'])
-
-                if sort:
-                    res_json = self.sort_response_by_fid(
-                        res_json, sort, not ascending)
-
-                data = res_json.get("data")
+                data = self.fix_nums(response.get('data'))
 
                 if round_ints:
                     data = self.round_ints(data)
@@ -316,10 +260,9 @@ class Client(object):
             except Exception as e:
                 print(traceback.format_exc())
 
+        # run a report with custom columns
         elif qid and columns:
-            print("qid and columns")
             try:
-                print("qid and columns")
                 url = f"https://api.quickbase.com/v1/reports/{qid}"
                 params = {
                     'tableId': database,
@@ -329,7 +272,7 @@ class Client(object):
                     params=params,
                     headers=self.headers)
                 response = response.json()
-                print(response)
+
                 fields = columns
                 where = response['query']['filter']
                 sortBy = response['query']['sortBy']
@@ -346,31 +289,15 @@ class Client(object):
                 if groupBy:
                     body.update({"groupBy": groupBy})
 
-                response = requests.post(
-                    'https://api.quickbase.com/v1/records/query',
-                    headers=self.headers,
-                    json=body
-                )
+                response = self.perform_query(body=body)
 
-                response = response.json()
-
-                response['data'] = self.fix_nums(response['data'])
-
-                if sort:
-                    response = self.sort_response_by_fid(response, sort)
+                data = self.fix_nums(response.get('data'))
 
                 if round_ints:
-                    response['data'] = self.round_ints(response['data'])
+                    data = self.round_ints(data)
 
-                return_records = []
-                records = response['data']
-                for record in records:
-                    record_dict = {}
-                    for field in columns:
-                        record_dict[str(field)] = record[str(field)]['value']
-                    return_records.append(record_dict)
+                return self.denest(data)
 
-                return return_records
 
             except Exception as e:
                 print(traceback.format_exc())
@@ -383,15 +310,7 @@ class Client(object):
             'top': max
         }, headers=self.headers)
 
-        return response
-
-    def sort_response_by_fid(self, response, sort, reverse=False) -> Dict[str, Any]:
-        """sorts response (json) by the fields provided (sort should be list of fid's)"""
-        for fid in sort:
-            response['data'] = sorted(
-                response['data'], key=lambda record: record[str(fid)]['value'], reverse=reverse)
-
-        return response
+        return response.json()
 
     def perform_query(self, body=None):
 
@@ -401,7 +320,7 @@ class Client(object):
             json=body
         )
 
-        return response
+        return response.json()
 
     def add_record(self, fields, database=None, uploads=None, safemode=False):
         """
@@ -552,7 +471,7 @@ class Client(object):
 
         return res_json.get("numberDeleted")
 
-    def get_file(self, url=None, database=None, record=None, field=None, version=1, user_agent=None):
+    def get_file(self, url=None, database=None, record=None, field=None, version=1):
         """
         fetches a file from a record
         params:
@@ -563,7 +482,9 @@ class Client(object):
         the latter will be used to create url
         optional params:
         version: version of the file attachment(in case the file has been replaced), defaults to version 1
-        user_agent: optional string, not entirely sure what this is for, but the api call has it
+
+        **Important, json endpoint returns base64 encoded file, will need to b64 decode the bytes
+
         """
 
         headers = self.headers
@@ -829,4 +750,4 @@ class Client(object):
             error_text = error_text.text if error_text is not None else '[no error text]'
             return f"Quickbase error: {error_text}"
 
-        return "Sucessfully changed record owner."
+        return "Successfully changed record owner."
