@@ -1,5 +1,5 @@
 import time
-from . import json_quickbase, xml_quickbase
+import json_quickbase, xml_quickbase
 from typing import Union, Dict
 import os
 import getpass
@@ -14,9 +14,9 @@ def get_client(json_sdk: bool = None, creds: Dict=None) -> Union[json_quickbase.
     load_dotenv(find_dotenv())
     if creds:
         url = creds.get('QB_URL')
-        user_token = creds.get('USER_TOKEN')
-        username = creds.get('USERNAME')
-        password = creds.get('PASSWORD')
+        user_token = creds.get('QB_USERTOKEN')
+        username = creds.get('QB_USERNAME')
+        password = creds.get('QB_PASSWORD')
     else:
         url = os.environ.get('QB_URL')
         user_token = os.environ.get('QB_USER_TOKEN')
